@@ -13,7 +13,7 @@ from webviz_subsurface.plugins._simulation_time_series._views._subplot_view._uti
     create_history_vectors_df,
 )
 
-from ..mocks.ensemble_summary_provider_dummy import EnsembleSummaryProviderDummy
+from ....mocks.ensemble_summary_provider_dummy import EnsembleSummaryProviderDummy
 
 # *******************************************************************
 #####################################################################
