@@ -1,13 +1,24 @@
-from pathlib import Path
-import warnings
 import glob
-from typing import Union, Optional, List
+import warnings
+from pathlib import Path
+from typing import List, Optional, Union
 
 import pandas as pd
-from fmu.ensemble import ScratchEnsemble, EnsembleSet
 from webviz_config.common_cache import CACHE
 from webviz_config.webviz_store import webvizstore
 
+# The fmu.ensemble dependency ecl is only available for Linux,
+# hence, ignore any import exception here to make
+# it still possible to use the PvtPlugin on
+# machines with other OSes.
+#
+# NOTE: Functions in this file cannot be used
+#       on non-Linux OSes.
+try:
+    from fmu.ensemble import EnsembleSet, ScratchEnsemble
+except ImportError:
+    pass
+
 
 @CACHE.memoize(timeout=CACHE.TIMEOUT)
 def scratch_ensemble(
