@@ -1,11 +1,15 @@
+import base64
+import io
 import json
 import math
+import zipfile
 from copy import deepcopy
 from typing import Any, Callable, Dict, List, Optional, Tuple, Union
 
 import numpy as np
 from dash import ALL, MATCH, Input, Output, State, callback, callback_context, no_update
 from dash.exceptions import PreventUpdate
+from webviz_config import EncodedFile
 from webviz_config.utils._dash_component_utils import calculate_slider_step
 
 from webviz_subsurface._providers import (
@@ -19,12 +23,12 @@ from webviz_subsurface._providers import (
     SimulatedSurfaceAddress,
     StatisticalSurfaceAddress,
     SurfaceAddress,
-    SurfaceServer,
+    SurfaceArrayServer,
 )
 
 from ._layer_model import DeckGLMapLayersModel
 from ._tmp_well_pick_provider import WellPickProvider
-from ._types import SurfaceMode
+from ._types import LayerTypes, SurfaceMode
 from .layout import (
     DefaultSettings,
     LayoutElements,
@@ -39,13 +43,15 @@ from .layout import (
 def plugin_callbacks(
     get_uuid: Callable,
     ensemble_surface_providers: Dict[str, EnsembleSurfaceProvider],
-    surface_server: SurfaceServer,
+    surface_server: SurfaceArrayServer,
     ensemble_fault_polygons_providers: Dict[str, EnsembleFaultPolygonsProvider],
     fault_polygons_server: FaultPolygonsServer,
     map_surface_names_to_fault_polygons: Dict[str, str],
     well_picks_provider: Optional[WellPickProvider],
     fault_polygon_attribute: Optional[str],
     color_tables: List[Dict],
+    plugin_data_output: Output,
+    plugin_data_requested: Input,
 ) -> None:
     def selections(tab: str, colorselector: bool = False) -> Dict[str, str]:
         uuid = get_uuid(
@@ -298,7 +304,7 @@ def plugin_callbacks(
     # 5th callback
     @callback(
         Output({"id": get_uuid(LayoutElements.DECKGLMAP), "tab": MATCH}, "layers"),
-        Output({"id": get_uuid(LayoutElements.DECKGLMAP), "tab": MATCH}, "bounds"),
+        # Output({"id": get_uuid(LayoutElements.DECKGLMAP), "tab": MATCH}, "bounds"),
         Output({"id": get_uuid(LayoutElements.DECKGLMAP), "tab": MATCH}, "views"),
         Input(
             {"id": get_uuid(LayoutElements.VERIFIED_VIEW_DATA), "tab": MATCH}, "data"
@@ -332,63 +338,47 @@ def plugin_callbacks(
             include_well_layer=well_picks_provider is not None,
             visible_well_layer=LayoutLabels.SHOW_WELLS in options,
             visible_fault_polygons_layer=LayoutLabels.SHOW_FAULTPOLYGONS in options,
-            visible_hillshading_layer=LayoutLabels.SHOW_HILLSHADING in options,
         )
         layer_model = DeckGLMapLayersModel(layers)
 
         for idx, data in enumerate(surface_elements):
             diff_surf = data.get("surf_type") == "diff"
-            surf_meta, img_url = (
+            surf_meta, mesh_url = (
                 get_surface_metadata_and_image(data)
                 if not diff_surf
                 else get_surface_metadata_and_image_for_diff_surface(surface_elements)
             )
-            viewport_bounds = [
-                surf_meta.x_min,
-                surf_meta.y_min,
-                surf_meta.x_max,
-                surf_meta.y_max,
-            ]
-
-            # Map3DLayer currently not implemented. Will replace
-            # ColormapLayer and HillshadingLayer
-            # layer_data = {
-            #     "mesh": img_url,
-            #     "propertyTexture": img_url,
-            #     "bounds": surf_meta.deckgl_bounds,
-            #     "rotDeg": surf_meta.deckgl_rot_deg,
-            #     "meshValueRange": [surf_meta.val_min, surf_meta.val_max],
-            #     "propertyValueRange": [surf_meta.val_min, surf_meta.val_max],
-            #     "colorMapName": data["colormap"],
-            #     "colorMapRange": data["color_range"],
-            # }
-
-            # layer_model.update_layer_by_id(
-            #     layer_id=f"{LayoutElements.MAP3D_LAYER}-{idx}",
-            #     layer_data=layer_data,
-            # )
+            if (
+                data["color_range"][0] != surf_meta.val_min
+                or data["color_range"][1] != surf_meta.val_max
+            ):
+                color_range = data["color_range"]
+            else:
+                color_range = None
             layer_data = {
-                "image": img_url,
-                "bounds": surf_meta.deckgl_bounds,
-                "rotDeg": surf_meta.deckgl_rot_deg,
-                "valueRange": [surf_meta.val_min, surf_meta.val_max],
-            }
-            layer_model.update_layer_by_id(
-                layer_id=f"{LayoutElements.COLORMAP_LAYER}-{idx}",
-                layer_data=layer_data,
-            )
-
-            layer_model.update_layer_by_id(
-                layer_id=f"{LayoutElements.HILLSHADING_LAYER}-{idx}",
-                layer_data=layer_data,
-            )
-            layer_model.update_layer_by_id(
-                layer_id=f"{LayoutElements.COLORMAP_LAYER}-{idx}",
-                layer_data={
-                    "colorMapName": data["colormap"],
-                    "colorMapRange": data["color_range"],
+                "meshUrl": mesh_url,
+                "frame": {
+                    "origin": [surf_meta.x_ori, surf_meta.y_ori],
+                    "count": [surf_meta.x_count, surf_meta.y_count],
+                    "increment": [surf_meta.x_inc, surf_meta.y_inc],
+                    "rotDeg": surf_meta.rot_deg,
                 },
-            )
+                "colorMapName": data["colormap"],
+                "colorMapRange": color_range,
+            }
+            layer_idx = None
+            for layer in layers:
+                if layer["id"] == f"{LayoutElements.MAP3D_LAYER}-{idx}":
+                    layer_idx = layers.index(layer)
+                    break
+            if layer_idx is not None:
+                layers[layer_idx].update(layer_data)
+            else:
+                layer_data["id"] = f"{LayoutElements.MAP3D_LAYER}-{idx}"
+                layer_data["@@type"] = LayerTypes.MAP3D
+                layer_data["material"] = False
+                layers.insert(0, layer_data)
+
             if (
                 LayoutLabels.SHOW_FAULTPOLYGONS in options
                 and fault_polygon_attribute is not None
@@ -429,7 +419,7 @@ def plugin_callbacks(
                 )
         return (
             layer_model.layers,
-            viewport_bounds if surface_elements else no_update,
+            # viewport_bounds if surface_elements else no_update,
             {
                 "layout": view_layout(len(surface_elements), view_columns),
                 "showLabel": True,
@@ -437,10 +427,9 @@ def plugin_callbacks(
                     {
                         "id": f"{view}_view",
                         "show3D": False,
+                        "isSync": True,
                         "layerIds": [
-                            # f"{LayoutElements.MAP3D_LAYER}-{view}",
-                            f"{LayoutElements.COLORMAP_LAYER}-{view}",
-                            f"{LayoutElements.HILLSHADING_LAYER}-{view}",
+                            f"{LayoutElements.MAP3D_LAYER}-{view}",
                             f"{LayoutElements.FAULTPOLYGONS_LAYER}-{view}",
                             f"{LayoutElements.WELLS_LAYER}-{view}",
                         ],
@@ -453,6 +442,77 @@ def plugin_callbacks(
             },
         )
 
+    @callback(
+        plugin_data_output,
+        plugin_data_requested,
+        State(
+            {"id": get_uuid(LayoutElements.VERIFIED_VIEW_DATA), "tab": ALL},
+            "data",
+        ),
+        State(get_uuid("tabs"), "value"),
+        State({"id": get_uuid(LayoutElements.VERIFIED_VIEW_DATA), "tab": ALL}, "id"),
+        prevent_initial_call=True,
+    )
+    def _user_download_data(
+        data_requested: bool,
+        surface_elements_in_tabs: List[List[Dict]],
+        tab_name: str,
+        tab_ids: List[dict],
+    ) -> Optional[EncodedFile]:
+        """Callback for downloading surfaces from the plugin"""
+        if not surface_elements_in_tabs or not data_requested:
+            return no_update
+        index = [x["tab"] for x in tab_ids].index(tab_name)
+        surface_elements_in_tab = surface_elements_in_tabs[index]
+        surface_bytes = {}
+        reals_in_statistics = []
+        for s_elem in surface_elements_in_tab:
+            if s_elem.get("surf_type") == "diff":
+                continue
+            surface_address = get_surface_address_from_data(s_elem)
+            ensemble = s_elem["ensemble"][0]
+            provider = ensemble_surface_providers[ensemble]
+            surface = provider.get_surface(surface_address)
+            if surface is None:
+                continue
+            date_str = s_elem["date"][0] if s_elem["date"] else ""
+            realizations = s_elem["realizations"]
+            mode_str = (
+                s_elem["mode"]
+                if SurfaceMode(s_elem["mode"]) != SurfaceMode.REALIZATION
+                else realizations[0]
+            )
+            surface_name = (
+                f"{s_elem['attribute'][0]}--{s_elem['name'][0]}"
+                f"--{date_str}--{mode_str}--{ensemble}.gri"
+            )
+            if SurfaceMode(s_elem["mode"]) not in [
+                SurfaceMode.OBSERVED,
+                SurfaceMode.REALIZATION,
+            ]:
+                reals_in_statistics.append(
+                    f"{surface_name},"
+                    f"INCLUDED REALIZATIONS: {realizations}, EXCLUDED REALIZATIONS: "
+                    f"{[real for real in provider.realizations() if real not in realizations]}"
+                )
+            byte_stream = io.BytesIO()
+            surface.to_file(byte_stream)
+            surface_bytes[surface_name] = byte_stream
+
+        with io.BytesIO() as bytes_io:
+            with zipfile.ZipFile(bytes_io, "w") as zipped_data:
+                for surface_name, surface_byte in surface_bytes.items():
+                    zipped_data.writestr(surface_name, surface_byte.getvalue())
+                zipped_data.writestr(
+                    "statistics.txt",
+                    "\n".join(str(real) for real in reals_in_statistics),
+                )
+            return {
+                "filename": "surfaces.zip",
+                "content": base64.b64encode(bytes_io.getvalue()).decode("ascii"),
+                "mime_type": "application/zip",
+            }
+
     def make_viewport_label(data: dict, tab: str, multi: Optional[str]) -> str:
         """Return text-label for each viewport based on which tab is selected"""
         # For the difference view
@@ -841,7 +901,7 @@ def view_layout(views: int, columns: Optional[int] = None) -> List[int]:
     columns = (
         columns
         if columns is not None
-        else min([x for x in range(20) if (x * x) >= views])
+        else min([x for x in range(1, 20, 1) if (x * x) >= views])
     )
     rows = math.ceil(views / columns)
     return [rows, columns]
