@@ -1,10 +1,11 @@
+# pylint: disable=too-many-lines
 import warnings
 from pathlib import Path
 from typing import Optional, Union
 
 import numpy as np
 import webviz_core_components as wcc
-from dash import Input, Output, State, dcc
+from dash import Input, Output, State, dcc, html
 from dash.exceptions import PreventUpdate
 from webviz_config import WebvizPluginABC, WebvizSettings
 from webviz_config.common_cache import CACHE
@@ -14,6 +15,7 @@ import webviz_subsurface
 
 from .._datainput.fmu_input import load_csv
 from .._datainput.relative_permeability import load_satfunc, load_scal_recommendation
+from .._utils.colors import StandardColors
 from .._utils.fanchart_plotting import (
     FanchartData,
     FreeLineData,
@@ -35,6 +37,10 @@ class RelativePermeability(WebvizPluginABC):
     relative to the `webviz` configuration.
 * **`sheet_name`:** Which sheet to use for the `scalfile`, only relevant if `scalfile` is an \
     `xlsx` file (recommended to use csv files with `webviz`).
+* **`scal_scenarios`:** Alternative to ensemble based input, using only a dict of where the \
+    keys are arbitrary names used for each scenario and the values are absolute path to \
+    pyscal files for each scenario (if using `scal_scenarios`, this has to be the **only** \
+    input option used).
 
 ---
 The minimum requirement is to define `ensembles`.
@@ -68,6 +74,9 @@ factory.PyscalFactory.load_relperm_df) input format. Including this file adds re
 `sheet_name` defines the sheet to use in the `scalfile`. Only relevant if `scalfile` is an
 `xlsx` file (it is recommended to use `csv` and not `xlsx` for `Webviz`).
 
+`scal_scenarios` is a dict with pyscal formatted files (similar to `scalfile`) as values,
+and arbitrary names for each scenario as keys.
+
 * [Example of relpermfile](https://github.com/equinor/webviz-subsurface-testdata/blob/master/\
 reek_history_match/realization-0/iter-0/share/results/tables/relperm.csv).
 * [Example of scalfile](https://github.com/equinor/\
@@ -81,22 +90,23 @@ webviz-subsurface-testdata/blob/master/reek_history_match/share/scal/scalreek.cs
     }
     SCAL_COLORMAP = {
         "Missing": "#ffff00",  # using yellow if the curve could not be found
-        "KRW": "#0000aa",
-        "KRG": "#ff0000",
-        "KROG": "#00aa00",
-        "KROW": "#00aa00",
+        "KRW": StandardColors.WATER_BLUE,
+        "KRG": StandardColors.GAS_RED,
+        "KROG": StandardColors.OIL_GREEN,
+        "KROW": StandardColors.OIL_GREEN,
         "PCOW": "#555555",  # Reserving #000000 for reference envelope (scal rec)
         "PCOG": "#555555",
     }
-
+    # pylint: disable=too-many-branches,too-many-statements
     def __init__(
         self,
         app,
         webviz_settings: WebvizSettings,
-        ensembles: list,
+        ensembles: Optional[list] = None,
         relpermfile: str = None,
         scalfile: Path = None,
         sheet_name: Optional[Union[str, int, list]] = None,
+        scal_scenarios: Optional[dict] = None,
     ):
 
         super().__init__()
@@ -107,113 +117,172 @@ webviz-subsurface-testdata/blob/master/reek_history_match/share/scal/scalreek.cs
             / "css"
             / "block_options.css"
         )
-
-        self.ens_paths = {
-            ens: webviz_settings.shared_settings["scratch_ensembles"][ens]
-            for ens in ensembles
-        }
         self.plotly_theme = webviz_settings.theme.plotly_theme
-        self.relpermfile = relpermfile
-        if self.relpermfile is not None:
-            self.satfunc = load_csv(ensemble_paths=self.ens_paths, csv_file=relpermfile)
-            self.satfunc = self.satfunc.rename(str.upper, axis="columns").rename(
-                columns={"TYPE": "KEYWORD"}
-            )
-            if "KEYWORD" not in self.satfunc.columns:
+        self.scalfile = scalfile
+        self.sheet_name = sheet_name
+        self.scal_scenarios = scal_scenarios
+        if scal_scenarios:
+            self.plugin_mode = "scal_scenarios"
+            if any((ensembles, relpermfile, scalfile, sheet_name)):
                 raise ValueError(
-                    "There has to be a KEYWORD or TYPE column with corresponding Eclipse keyword: "
-                    "e.g SWOF, SGOF and etc."
+                    "`scal_scenarios` can only be used alone as an argument."
                 )
-            valid_columns = (
-                ["ENSEMBLE", "REAL", "KEYWORD", "SATNUM"]
-                + RelativePermeability.SATURATIONS
-                + [
-                    key
-                    for key in RelativePermeability.SCAL_COLORMAP
-                    if key != "Missing"
-                ]
-            )
-            self.satfunc = self.satfunc[
-                [col for col in self.satfunc.columns if col in valid_columns]
-            ]
+            self.ensembles = []
+            self.scal_ensembles = {}
+            self.sat_axes_maps = {}
+            satnums = set()
+
+            for case, path in scal_scenarios.items():
+                self.ensembles.append(case)
+                self.scal_ensembles[case] = load_scal_recommendation(Path(path))
+                satnums.update(self.scal_ensembles[case]["SATNUM"].unique())
+                for sat_axis in RelativePermeability.SATURATIONS:
+                    if sat_axis in self.scal_ensembles[case]:
+                        if sat_axis not in self.sat_axes_maps:
+                            self.sat_axes_maps[sat_axis] = set()
+                        if sat_axis == "SW":
+                            self.sat_axes_maps[sat_axis].update(
+                                [
+                                    col
+                                    for col in self.scal_ensembles[case].columns
+                                    if col in ["KRW", "KROW", "PCOW"]
+                                ]
+                            )
+                        if sat_axis == "SG":
+                            self.sat_axes_maps[sat_axis].update(
+                                [
+                                    col
+                                    for col in self.scal_ensembles[case].columns
+                                    if col in ["KRG", "KROG", "PCOG"]
+                                ]
+                            )
+                        if sat_axis == "SL":
+                            self.sat_axes_maps[sat_axis].update(
+                                [
+                                    col
+                                    for col in self.scal_ensembles[case].columns
+                                    if col in ["KRG", "KROG", "PCOG"]
+                                ]
+                            )
+                        if sat_axis == "SO":
+                            self.sat_axes_maps[sat_axis].update(
+                                [
+                                    col
+                                    for col in self.scal_ensembles[case].columns
+                                    if col in ["KROW", "KROG"]
+                                ]
+                            )
+            self.sat_axes_maps = {
+                sat_axis: list(cols) for sat_axis, cols in self.sat_axes_maps.items()
+            }
+            self.satnums = sorted(satnums)
         else:
-            self.satfunc = load_satfunc(self.ens_paths)
-        if any(
-            keyword in RelativePermeability.RELPERM_FAMILIES[1]
-            for keyword in self.satfunc["KEYWORD"].unique()
-        ):
-            self.family = 1
+            self.plugin_mode = "distribution"
+            self.ens_paths = {
+                ens: webviz_settings.shared_settings["scratch_ensembles"][ens]
+                for ens in ensembles
+            }
+            self.ensembles = ensembles
+            self.relpermfile = relpermfile
+            if self.relpermfile is not None:
+                self.satfunc = load_csv(
+                    ensemble_paths=self.ens_paths, csv_file=relpermfile
+                )
+                self.satfunc = self.satfunc.rename(str.upper, axis="columns").rename(
+                    columns={"TYPE": "KEYWORD"}
+                )
+                if "KEYWORD" not in self.satfunc.columns:
+                    raise ValueError(
+                        "There has to be a KEYWORD or TYPE column with corresponding Eclipse "
+                        "keyword: e.g SWOF, SGOF and etc."
+                    )
+                valid_columns = (
+                    ["ENSEMBLE", "REAL", "KEYWORD", "SATNUM"]
+                    + RelativePermeability.SATURATIONS
+                    + [
+                        key
+                        for key in RelativePermeability.SCAL_COLORMAP
+                        if key != "Missing"
+                    ]
+                )
+                self.satfunc = self.satfunc[
+                    [col for col in self.satfunc.columns if col in valid_columns]
+                ]
+            else:
+                self.satfunc = load_satfunc(self.ens_paths)
             if any(
+                keyword in RelativePermeability.RELPERM_FAMILIES[1]
+                for keyword in self.satfunc["KEYWORD"].unique()
+            ):
+                self.family = 1
+                if any(
+                    keyword in RelativePermeability.RELPERM_FAMILIES[2]
+                    for keyword in self.satfunc["KEYWORD"].unique()
+                ):
+                    warnings.warn(
+                        (
+                            "Mix of keyword family 1 and 2, currently only support one family at "
+                            "the time. Dropping all data of family 2 ('SWFN', 'SGFN', 'SGWFN', "
+                            "'SOF2', 'SOF3', 'SOF32D') and continues with family 1 ('SWOF', "
+                            "'SGOF', 'SLGOF')."
+                        ),
+                    )
+                    self.satfunc = self.satfunc[
+                        self.satfunc["KEYWORD"].isin(
+                            RelativePermeability.RELPERM_FAMILIES["fam1"]
+                        )
+                    ]
+                if "SGOF" in self.satfunc["KEYWORD"].unique():
+                    if "SLGOF" in self.satfunc["KEYWORD"].unique():
+                        warnings.warn(
+                            (
+                                "Mix of 'SGOF' and 'SLGOF' in ensembles, resulting in non-unique "
+                                "horizontal axis ('SG' and 'SL') for 'KRG', 'KROG' and 'PCOG'. "
+                                "Dropping all data with 'SLGOF'."
+                            ),
+                        )
+                        self.satfunc = self.satfunc[self.satfunc["KEYWORD"] != "SLGOF"]
+                    self.sat_axes_maps = {
+                        "SW": ["KRW", "KROW", "PCOW"],
+                        "SG": ["KRG", "KROG", "PCOG"],
+                    }
+                else:
+                    self.sat_axes_maps = {
+                        "SW": ["KRW", "KROW", "PCOW"],
+                        "SL": ["KRG", "KROG", "PCOG"],
+                    }
+            elif not all(
                 keyword in RelativePermeability.RELPERM_FAMILIES[2]
                 for keyword in self.satfunc["KEYWORD"].unique()
             ):
-                warnings.warn(
-                    (
-                        "Mix of keyword family 1 and 2, currently only support one family at the "
-                        "time. Dropping all data of family 2 ('SWFN', 'SGFN', 'SGWFN', 'SOF2', "
-                        "'SOF3', 'SOF32D') and continues with family 1 ('SWOF', 'SGOF', 'SLGOF')."
-                    ),
+                raise ValueError(
+                    "Unrecognized saturation table keyword in data. This should not occur unless "
+                    "there has been changes to ecl2df. Update of this plugin might be required."
                 )
-                self.satfunc = self.satfunc[
-                    self.satfunc["KEYWORD"].isin(
-                        RelativePermeability.RELPERM_FAMILIES["fam1"]
-                    )
-                ]
-            if "SGOF" in self.satfunc["KEYWORD"].unique():
-                if "SLGOF" in self.satfunc["KEYWORD"].unique():
-                    warnings.warn(
-                        (
-                            "Mix of 'SGOF' and 'SLGOF' in ensembles, resulting in non-unique "
-                            "horizontal axis ('SG' and 'SL') for 'KRG', 'KROG' and 'PCOG'. "
-                            "Dropping all data with 'SLGOF'."
-                        ),
-                    )
-                    self.satfunc = self.satfunc[self.satfunc["KEYWORD"] != "SLGOF"]
-                self.sat_axes_maps = {
-                    "SW": ["KRW", "KROW", "PCOW"],
-                    "SG": ["KRG", "KROG", "PCOG"],
-                }
             else:
+                self.family = 2
                 self.sat_axes_maps = {
-                    "SW": ["KRW", "KROW", "PCOW"],
-                    "SL": ["KRG", "KROG", "PCOG"],
+                    "SW": ["KRW", "PCOW"],
+                    "SG": ["KRG", "PCOG"],
+                    "SO": ["KROW", "KROG"],
                 }
-        elif not all(
-            keyword in RelativePermeability.RELPERM_FAMILIES[2]
-            for keyword in self.satfunc["KEYWORD"].unique()
-        ):
-            raise ValueError(
-                "Unrecognized saturation table keyword in data. This should not occur unless "
-                "there has been changes to ecl2df. Update of this plugin might be required."
+            self.scal_ensembles = (
+                {"SCAL": load_scal_recommendation(self.scalfile, self.sheet_name)}
+                if self.scalfile is not None
+                else None
             )
-        else:
-            self.family = 2
-            self.sat_axes_maps = {
-                "SW": ["KRW", "PCOW"],
-                "SG": ["KRG", "PCOG"],
-                "SO": ["KROW", "KROG"],
-            }
-        self.scalfile = scalfile
-        self.sheet_name = sheet_name
-        self.scal = (
-            load_scal_recommendation(self.scalfile, self.sheet_name)
-            if self.scalfile is not None
-            else None
-        )
+            self.satnums = sorted(self.satfunc["SATNUM"].unique())
+
         self.set_callbacks(app)
 
     @property
     def sat_axes(self):
         """List of all possible saturation axes in dataframe"""
-        return [sat for sat in self.sat_axes_maps if sat in self.satfunc.columns]
-
-    @property
-    def ensembles(self):
-        return list(self.satfunc["ENSEMBLE"].unique())
-
-    @property
-    def satnums(self):
-        return list(self.satfunc["SATNUM"].unique())
+        return (
+            list(self.sat_axes_maps)
+            if self.scal_ensembles
+            else [sat for sat in self.sat_axes_maps if sat in self.satfunc.columns]
+        )
 
     @property
     def color_options(self):
@@ -385,21 +454,42 @@ webviz-subsurface-testdata/blob/master/reek_history_match/share/scal/scalreek.cs
                         wcc.Selectors(
                             label="Visualization",
                             children=[
-                                wcc.RadioItems(
-                                    label="Line traces",
-                                    id=self.uuid("visualization"),
-                                    className="block-options",
-                                    options=[
-                                        {
-                                            "label": "Individual realizations",
-                                            "value": "realizations",
-                                        },
-                                        {
-                                            "label": "Statistical fanchart",
-                                            "value": "statistics",
-                                        },
-                                    ],
-                                    value="statistics",
+                                html.Div(
+                                    wcc.RadioItems(
+                                        label="Line traces",
+                                        id=self.uuid("visualization"),
+                                        className="block-options",
+                                        options=[
+                                            {
+                                                "label": "Individual realizations",
+                                                "value": "realizations",
+                                            },
+                                            {
+                                                "label": "Statistical fanchart",
+                                                "value": "statistics",
+                                            },
+                                        ],
+                                        value="statistics",
+                                    ),
+                                    style={"display": "none"}
+                                    if self.plugin_mode == "scal_scenarios"
+                                    else {"display": "block"},
+                                ),
+                                html.Div(
+                                    wcc.Checklist(
+                                        label="SCAL recommendation",
+                                        id=self.uuid("scal"),
+                                        options=[
+                                            {"label": val.capitalize(), "value": val}
+                                            for val in ["pess", "base", "opt"]
+                                        ],
+                                        value=["pess", "base", "opt"]
+                                        if self.scal_ensembles is not None
+                                        else [],
+                                    ),
+                                    style={"display": "block"}
+                                    if self.scal_ensembles is not None
+                                    else {"display": "none"},
                                 ),
                                 wcc.RadioItems(
                                     label="Y-axis",
@@ -411,7 +501,7 @@ webviz-subsurface-testdata/blob/master/reek_history_match/share/scal/scalreek.cs
                                             "value": "linear",
                                         },
                                         {
-                                            "label": "Log",
+                                            "label": "Log (only kr)",
                                             "value": "log",
                                         },
                                     ],
@@ -419,27 +509,6 @@ webviz-subsurface-testdata/blob/master/reek_history_match/share/scal/scalreek.cs
                                 ),
                             ],
                         ),
-                        wcc.Selectors(
-                            style={"display": "block"}
-                            if self.scal is not None
-                            else {"display": "none"},
-                            id=self.uuid("scal_selector"),
-                            label="SCAL recommendation",
-                            children=[
-                                wcc.Checklist(
-                                    id=self.uuid("scal"),
-                                    options=[
-                                        {
-                                            "label": "Show SCAL",
-                                            "value": "show_scal",
-                                        },
-                                    ],
-                                    value=["show_scal"]
-                                    if self.scal is not None
-                                    else [],
-                                ),
-                            ],
-                        ),
                     ],
                 ),
                 wcc.Frame(
@@ -466,7 +535,14 @@ webviz-subsurface-testdata/blob/master/reek_history_match/share/scal/scalreek.cs
             ],
         )
         def _update_graph(
-            color_by, visualization, ensembles, curves, satnums, sataxis, linlog, scal
+            color_by,
+            visualization,
+            ensembles,
+            curves,
+            satnums,
+            sataxis,
+            linlog,
+            selected_scal,
         ):
             if not curves or not satnums:  # Curve and satnum has to be defined
                 raise PreventUpdate
@@ -478,7 +554,6 @@ webviz-subsurface-testdata/blob/master/reek_history_match/share/scal/scalreek.cs
                 curves = [curves]
             if not isinstance(satnums, list):
                 satnums = [satnums]
-            df = filter_df(self.satfunc, ensembles, curves, satnums, sataxis)
             if color_by == "ENSEMBLE":
                 colors = self.ens_colors
             elif color_by == "SATNUM":
@@ -494,20 +569,67 @@ webviz-subsurface-testdata/blob/master/reek_history_match/share/scal/scalreek.cs
             layout = plot_layout(
                 nplots, curves, sataxis, color_by, linlog, self.plotly_theme["layout"]
             )
-            if visualization == "realizations" and not df.empty:
-                traces = add_realization_traces(
-                    df, color_by, curves, sataxis, colors, nplots
-                )
-            elif visualization == "statistics" and not df.empty:
-                traces = add_statistic_traces(
-                    df, color_by, curves, sataxis, colors, nplots
-                )
-            else:
-                traces = []
+            traces = []
+            if self.plugin_mode == "distribution":
+                df = filter_df(self.satfunc, ensembles, curves, satnums, sataxis)
+                if visualization == "realizations" and not df.empty:
+                    traces.extend(
+                        add_realization_traces(
+                            df, color_by, curves, sataxis, colors, nplots
+                        )
+                    )
+                elif visualization == "statistics" and not df.empty:
+                    traces.extend(
+                        add_statistic_traces(
+                            df, color_by, curves, sataxis, colors, nplots
+                        )
+                    )
+
+            if selected_scal:
+                if self.plugin_mode == "distribution":
+                    scal_df = filter_scal_df(
+                        self.scal_ensembles["SCAL"],
+                        curves,
+                        satnums,
+                        sataxis,
+                        selected_scal,
+                    )
+                    traces.extend(
+                        add_scal_traces(
+                            scal_df,
+                            color_by,
+                            curves,
+                            sataxis,
+                            colors,
+                            nplots,
+                            "SCAL",
+                            self.plugin_mode,
+                        )
+                    )
+                elif self.plugin_mode == "scal_scenarios":
+                    for scal_scenario in ensembles:
+                        scal_df = filter_scal_df(
+                            self.scal_ensembles[scal_scenario],
+                            curves,
+                            satnums,
+                            sataxis,
+                            selected_scal,
+                        )
+                        traces.extend(
+                            add_scal_traces(
+                                scal_df,
+                                color_by,
+                                curves,
+                                sataxis,
+                                colors,
+                                nplots,
+                                scal_scenario,
+                                self.plugin_mode,
+                            )
+                        )
+                else:
+                    raise ValueError(f"Invalid plugin_mode: {self.plugin_mode}")
 
-            if self.scal is not None and "show_scal" in scal:
-                scal_df = filter_scal_df(self.scal, curves, satnums, sataxis)
-                traces.extend(add_scal_traces(scal_df, curves, sataxis, nplots))
             return {"data": traces, "layout": layout}
 
         @app.callback(
@@ -575,6 +697,15 @@ webviz-subsurface-testdata/blob/master/reek_history_match/share/scal/scalreek.cs
             )
 
     def add_webvizstore(self):
+        if self.scal_scenarios:
+            return [
+                (
+                    load_scal_recommendation,
+                    [{"scalfile": Path(path)}],
+                )
+                for _, path in self.scal_scenarios.items()
+            ]
+
         return [
             (
                 load_satfunc,
@@ -608,8 +739,6 @@ webviz-subsurface-testdata/blob/master/reek_history_match/share/scal/scalreek.cs
         )
 
 
-# Caching should be safe here with DataFrame as it is always the same for an instance of the plugin.
-@CACHE.memoize(timeout=CACHE.TIMEOUT)
 def filter_df(df, ensembles, curves, satnums, sataxis):
     df = df.copy()
     df = df.loc[df["ENSEMBLE"].isin(ensembles)]
@@ -618,11 +747,10 @@ def filter_df(df, ensembles, curves, satnums, sataxis):
     return df[columns].dropna()
 
 
-# Caching should be safe here with DataFrame as it is always the same for an instance of the plugin.
-@CACHE.memoize(timeout=CACHE.TIMEOUT)
-def filter_scal_df(df, curves, satnums, sataxis):
+def filter_scal_df(df, curves, satnums, sataxis, selected_scal):
     df = df.copy()
     df = df.loc[df["SATNUM"].isin(satnums)]
+    df = df.loc[df["CASE"].isin(selected_scal)]
     columns = (
         ["SATNUM", "CASE"]
         + [sataxis]
@@ -700,37 +828,133 @@ def add_realization_traces(df, color_by, curves, sataxis, colors, nplots):
     return traces
 
 
-def add_scal_traces(df, curves, sataxis, nplots):
+# pylint: disable=too-many-locals
+def add_scal_traces(
+    df, color_by, curves, sataxis, colors, nplots, scenario_name, plugin_mode
+):
     """Renders scal recommendation traces"""
     traces = []
+    linestyle = {"pess": "dash", "base": "solid", "opt": "dot"}
     for curve_no, curve in enumerate(
         [curve for curve in curves if curve in df.columns]
     ):
         yaxis = "y" if nplots == 1 or curve.startswith("KR") else "y2"
         xaxis = "x" if yaxis == "y" else "x2"
-        traces.extend(
-            [
-                {
-                    "type": "scatter",
-                    "x": case_df[sataxis],
-                    "y": case_df[curve],
-                    "xaxis": xaxis,
-                    "yaxis": yaxis,
-                    "hovertext": (
-                        f"{curve}, Satnum: {satnum}<br>" f"{case.lower().capitalize()}"
-                    ),
-                    "name": "SCAL",
-                    "legendgroup": "SCAL",
-                    "marker": {
-                        "color": "black",
-                    },
-                    "line": {"dash": "dash"},
-                    "showlegend": curve_no == 0 and satnum_no == 0 and case_no == 0,
-                }
-                for satnum_no, (satnum, satnum_df) in enumerate(df.groupby("SATNUM"))
-                for case_no, (case, case_df) in enumerate(satnum_df.groupby("CASE"))
-            ]
-        )
+        if plugin_mode == "scal_scenarios":
+            if color_by == "CURVE":
+                satnum = df["SATNUM"].iloc[0]
+                traces.extend(
+                    [
+                        {
+                            "type": "scatter",
+                            "x": case_df[sataxis],
+                            "y": case_df[curve],
+                            "xaxis": xaxis,
+                            "yaxis": yaxis,
+                            "hovertext": (
+                                f"{curve}, Satnum: {satnum}<br>"
+                                f"{case.capitalize()}, scenario: {scenario_name}"
+                            ),
+                            "name": curve,
+                            "legendgroup": curve,
+                            "marker": {
+                                "color": colors.get(
+                                    curve, colors[list(colors.keys())[0]]
+                                )
+                            },
+                            "line": {"dash": linestyle.get(case, "solid")},
+                            "showlegend": case_no == 0,
+                        }
+                        for case_no, (case, case_df) in enumerate(df.groupby("CASE"))
+                    ]
+                )
+            elif color_by == "SATNUM":
+                traces.extend(
+                    [
+                        {
+                            "type": "scatter",
+                            "x": case_df[sataxis],
+                            "y": case_df[curve],
+                            "xaxis": xaxis,
+                            "yaxis": yaxis,
+                            "hovertext": (
+                                f"{curve}, Satnum: {group}<br>"
+                                f"{case.capitalize()}, Ensemble: {scenario_name}"
+                            ),
+                            "name": group,
+                            "legendgroup": group,
+                            "marker": {
+                                "color": colors.get(
+                                    group, colors[list(colors.keys())[-1]]
+                                )
+                            },
+                            "line": {"dash": linestyle.get(case, "solid")},
+                            "showlegend": case_no == 0 and curve_no == 0,
+                        }
+                        for (group, grouped_df) in df.groupby(color_by)
+                        for case_no, (case, case_df) in enumerate(
+                            grouped_df.groupby("CASE")
+                        )
+                    ]
+                )
+            else:
+                constant_group = df["SATNUM"].iloc[0]
+                traces.extend(
+                    [
+                        {
+                            "type": "scatter",
+                            "x": case_df[sataxis],
+                            "y": case_df[curve],
+                            "xaxis": xaxis,
+                            "yaxis": yaxis,
+                            "hovertext": (
+                                f"{curve}, Satnum: {constant_group}<br>"
+                                f"{case.capitalize()}, Ensemble: {scenario_name}"
+                            ),
+                            "name": scenario_name,
+                            "legendgroup": scenario_name,
+                            "marker": {
+                                "color": colors.get(
+                                    scenario_name, colors[list(colors.keys())[-1]]
+                                )
+                            },
+                            "line": {"dash": linestyle.get(case, "solid")},
+                            "showlegend": case_no == 0 and curve_no == 0,
+                        }
+                        for case_no, (case, case_df) in enumerate(df.groupby("CASE"))
+                    ]
+                )
+
+        ##
+        elif plugin_mode == "distribution":
+            traces.extend(
+                [
+                    {
+                        "type": "scatter",
+                        "x": case_df[sataxis],
+                        "y": case_df[curve],
+                        "xaxis": xaxis,
+                        "yaxis": yaxis,
+                        "hovertext": (
+                            f"{curve}, Satnum: {satnum}<br>"
+                            f"{case.lower().capitalize()}"
+                        ),
+                        "name": scenario_name,
+                        "legendgroup": scenario_name,
+                        "marker": {
+                            "color": "black",
+                        },
+                        "line": {"dash": linestyle.get(case, "solid")},
+                        "showlegend": curve_no == 0 and satnum_no == 0 and case_no == 0,
+                    }
+                    for satnum_no, (satnum, satnum_df) in enumerate(
+                        df.groupby("SATNUM")
+                    )
+                    for case_no, (case, case_df) in enumerate(satnum_df.groupby("CASE"))
+                ]
+            )
+        else:
+            raise ValueError(f"Invalid plugin_mode: {plugin_mode}")
     return traces
 
 
@@ -746,11 +970,17 @@ def add_statistic_traces(df, color_by, curves, sataxis, colors, nplots):
 
     traces = []
     for ens_no, (ens, ens_df) in enumerate(
-        df[["ENSEMBLE", "SATNUM"] + [sataxis] + curves].groupby(["ENSEMBLE"])
+        df[["ENSEMBLE", "REAL", "SATNUM", sataxis] + curves].groupby(["ENSEMBLE"])
     ):
-        for satnum_no, (satnum, satnum_df) in enumerate(ens_df.groupby("SATNUM")):
+        for satnum_no, (satnum, satnum_df) in enumerate(
+            ens_df[["REAL", "SATNUM", sataxis] + curves].groupby("SATNUM")
+        ):
+            satnum_df_shared_axis = satnum_df[
+                satnum_df.groupby(sataxis)["REAL"].transform("nunique")
+                == satnum_df["REAL"].nunique()
+            ]
             df_stat = (
-                satnum_df.groupby(sataxis)
+                satnum_df_shared_axis.groupby(sataxis)
                 .agg([np.nanmean, np.nanmin, np.nanmax, p10, p90])
                 .stack()
                 .swaplevel()
@@ -830,7 +1060,7 @@ def _get_fanchart_traces(
     )
 
 
-@CACHE.memoize(timeout=CACHE.TIMEOUT)
+@CACHE.memoize()
 def plot_layout(nplots, curves, sataxis, color_by, linlog, theme):
     """
     Constructing plot layout from scratch as it is more responsive than plotly subplots package.
@@ -842,8 +1072,7 @@ def plot_layout(nplots, curves, sataxis, color_by, linlog, theme):
         if any(curve.startswith("KR") for curve in curves)
         else ["Capillary Pressure"]
     )
-    layout = {}
-    layout.update(theme)
+    layout = theme.copy()
     layout.update(
         {
             "hovermode": "closest",
@@ -913,6 +1142,7 @@ def plot_layout(nplots, curves, sataxis, color_by, linlog, theme):
                     "title": {"text": sataxis.lower().capitalize(), "standoff": 15},
                     "showgrid": False,
                     "tickmode": "auto",
+                    "exponentformat": "e",
                 },
                 "yaxis": {
                     "automargin": True,
@@ -920,16 +1150,16 @@ def plot_layout(nplots, curves, sataxis, color_by, linlog, theme):
                     "zeroline": False,
                     "anchor": "x",
                     "domain": [0.0, 1.0],
-                    "type": linlog,
                     "showgrid": False,
+                    "exponentformat": "e",
                 },
                 "margin": {"t": 20, "b": 0},
             }
         )
         if any(curve.startswith("KR") for curve in curves):
-            layout["yaxis"].update({"title": {"text": "kr"}})
+            layout["yaxis"].update({"title": {"text": "kr"}, "type": linlog})
         else:
-            layout["yaxis"].update({"title": {"text": "Pc"}})
+            layout["yaxis"].update({"title": {"text": "Pc"}, "type": "linear"})
 
     elif nplots == 2:
         layout.update(
@@ -942,6 +1172,7 @@ def plot_layout(nplots, curves, sataxis, color_by, linlog, theme):
                     "matches": "x2",
                     "showticklabels": False,
                     "showgrid": False,
+                    "exponentformat": "e",
                 },
                 "xaxis2": {
                     "automargin": True,
@@ -953,6 +1184,7 @@ def plot_layout(nplots, curves, sataxis, color_by, linlog, theme):
                     "domain": [0.0, 1.0],
                     "title": {"text": sataxis.lower().capitalize()},
                     "showgrid": False,
+                    "exponentformat": "e",
                 },
                 "yaxis": {
                     "automargin": True,
@@ -963,6 +1195,7 @@ def plot_layout(nplots, curves, sataxis, color_by, linlog, theme):
                     "title": {"text": "kr"},
                     "type": linlog,
                     "showgrid": False,
+                    "exponentformat": "e",
                 },
                 "yaxis2": {
                     "automargin": True,
@@ -971,8 +1204,9 @@ def plot_layout(nplots, curves, sataxis, color_by, linlog, theme):
                     "anchor": "x2",
                     "domain": [0.0, 0.475],
                     "title": {"text": "Pc"},
-                    "type": linlog,
+                    "type": "linear",
                     "showgrid": False,
+                    "exponentformat": "e",
                 },
                 "margin": {"t": 20, "b": 0},
             }
