@@ -2,5 +2,5 @@ from .clientside_stores import clientside_stores
 from .intersection_and_map import intersection_and_map_layout
 from .intersection_data import intersection_data_layout
 from .map_data import map_data_layout
-from .realization_modal import realization_layout
-from .uncertainty_table import uncertainty_table_btn, uncertainty_table_layout
+from .realization_filter import realization_layout
+from .uncertainty_table import uncertainty_table_layout
